@@ -3,15 +3,19 @@
 
 import logging
 import os
+from tqdm import tqdm
 from dataclasses import dataclass
 from enum import Enum
-from typing import List, Optional, Union
+from re import DEBUG, sub
+from shutil import Error
+from typing import List, Optional, Union, Dict
 
 import numpy as np
 
 import jsonlines
 
 from transformers import PreTrainedTokenizer, is_torch_available, BatchEncoding
+from transformers.utils.dummy_pt_objects import DebertaForQuestionAnswering
 
 logger = logging.getLogger(__name__)
 
@@ -178,16 +182,11 @@ class InputFeatures:
 
 @dataclass
 class InputFeatures_Interactive:
-    input_ids_seq: List[int]
-    attention_mask_seq: List[int]
-    token_type_ids_seq: Optional[List[int]]
-    # label_ids_seq: Optional[List[int]]
+    input_ids: List[int] = None
+    attention_mask: List[int] = None
+    token_type_ids: List[int] = None
+    triples: List[List[int]] = None
 
-    input_ids_ner: List[int]
-    attention_mask_ner: List[int]
-    token_type_ids_ner: Optional[List[int]]
-    label_ids_ner: Optional[List[int]]
-    label_ids: List = None
 
 
 class Split(Enum):
@@ -218,6 +217,9 @@ if is_torch_available():
             task='seq'
 
         ):
+        
+            with open(f"{data_dir}/rel2id.json", "r") as file:
+                rel2id = json.load(file)
             # Load data features from cache or dataset file
             cached_examples_file = os.path.join(
                 data_dir, "cached_{}_{}.examples".format(mode.value, tokenizer.__class__.__name__),
@@ -293,15 +295,7 @@ if is_torch_available():
                         labels_ner=labels_ner,
                         max_seq_length=max_seq_length,
                         tokenizer=tokenizer,
-                        cls_token_at_end=bool(model_type in ["xlnet"]),
-                        cls_token=tokenizer.cls_token,
-                        cls_token_segment_id=2 if model_type in ["xlnet"] else 0,
-                        sep_token=tokenizer.sep_token,
-                        sep_token_extra=False,
-                        pad_on_left=bool(tokenizer.padding_side == "left"),
-                        pad_token=tokenizer.pad_token_id,
-                        pad_token_segment_id=tokenizer.pad_token_type_id,
-                        pad_token_label_id=self.pad_token_label_id,
+                        rel2id=rel2id
                     )
 
                 logger.info(f"Saving features into cached file {cached_features_file}")
@@ -326,7 +320,7 @@ def read_examples_from_file(data_dir, mode: Union[Split, str]) -> List[InputExam
         text_id = 0
         
         for line in f.readlines():
-            item = json.loads(line)
+            item = eval(line)
             text = item['text']
             triples = []
             for triple in item['spo_list']:
@@ -358,10 +352,11 @@ def convert_examples_to_seq_features(
 ):
     features = []
     label2id = {label: i for i, label in enumerate(labels_seq)}
+    cnt = 0
 
-    for (ex_index, example) in enumerate(examples):
+    for (ex_index, example) in tqdm(enumerate(examples), total=len(examples)):
         inputs = tokenizer(
-            example.words,
+            preprocess(example.words),
             add_special_tokens=True,
             # return_overflowing_tokens=True,
             truncation="longest_first",
@@ -369,18 +364,26 @@ def convert_examples_to_seq_features(
         )
         label_ids_seq = []
         for triple in example.triples:
-            # input_ids: List[int]
-            # attention_mask: List[int]
-            # token_type_ids: Optional[List[int]] = None
-            # label_ids_seq: Optional[List[int]] = None
-            # label_ids_ner: Optional[List[int]] = None
             label_ids_seq.append(label2id[triple[1]])
+        if len(label_ids_seq) == 0:
+            cnt += 1
+            continue
+        label_ids_seq = torch.sum(torch.nn.functional.one_hot(torch.tensor(label_ids_seq), num_classes=len(labels_seq)), dim=0).float()
+        # the relation may show more than once, [1,2,1,0] -> [1,1,1,0]
+        label_ids_seq[label_ids_seq>0] = 1
 
         features.append(InputFeatures(input_ids=inputs['input_ids'], attention_mask=inputs['attention_mask'],
                                           token_type_ids=inputs['token_type_ids'], label_ids_seq=label_ids_seq))
 
+    logger.warning(f"total {cnt} samples error in the dataset!")
     return features
 
+
+
+def preprocess(text):
+    return text.lower()
+    return " ".join([_ for _ in text]).lower()
+
 def convert_examples_to_ner_features(
     examples: List[InputExample],
     labels_seq: List[str],
@@ -399,7 +402,10 @@ def convert_examples_to_ner_features(
     sequence_a_segment_id=0,
     mask_padding_with_zero=True,
 ):
+    # 将relation ids转化为特殊字符对应的ids,避免了relation 表示和原来的词表进行冲突
+    start_idx = tokenizer("[relation0]", add_special_tokens=False)['input_ids'][0]
     label_map_seq = {label: i for i, label in enumerate(labels_seq)}
+    seq_label2ids = {label: i+start_idx for i, label in enumerate(labels_seq)}
     label_map_ner = {label: i for i, label in enumerate(labels_ner)}
 
     features = []
@@ -408,9 +414,9 @@ def convert_examples_to_ner_features(
     def find_word_in_texts(word_ids, texts_ids):
         length = len(word_ids)
         for i, W in enumerate(texts_ids):
-            if texts_ids[i] == word_ids[0]:
-                if texts_ids[i: i + length] == word_ids:
-                    return i, i + length
+            if (i+length) >= len(texts_ids): break
+            if texts_ids[i: i + length] == word_ids:
+                return i, i + length
         return None, None
 
     for (ex_index, example) in enumerate(examples):
@@ -418,12 +424,35 @@ def convert_examples_to_ner_features(
         # text = example.text
         if ex_index % 10_000 == 0:
             logger.info("Writing example %d of %d", ex_index, len(examples))
+        
 
         text = example.words
+        
+        tmp_triples = {}
         for triple in example.triples:
-            subject = triple[0]
+            subject = preprocess(triple[0])
             relation = triple[1]
-            object_ = triple[2]
+            object_ = preprocess(triple[2])
+            if relation not in tmp_triples:
+                tmp_triples[relation] = [[subject, relation, object_]]
+            else:
+                tmp_triples[relation].append([subject, relation, object_])
+        
+        my_triples = []
+        for k, v in tmp_triples.items():
+            my_triples.append([[v[i][0] for i in range(len(v))], k, [v[i][2] for i in range(len(v))]])
+        
+        
+        hard_to_solve = 0
+        #  triple [[subject list], r, [object list]]
+        for triple in my_triples:
+            subject_list = triple[0]
+            relation = triple[1]
+            object_list = triple[2]
+            
+            # same entity map as subject and object            
+            if set(subject_list) & set(object_list) :
+                hard_to_solve += 1
 
             # cls w1 w2 .. sep w3 w4 sep 000000000
             # token_type
@@ -431,16 +460,16 @@ def convert_examples_to_ner_features(
             # 转换为id，加上cls以及seq等
             # {"input_ids":[], "token_type_ids":[], "attention_mask":[]}
             inputs = tokenizer(
-                text,
+                preprocess(text),
                 add_special_tokens=True,
                 max_length=max_seq_length-2,
                 truncation="longest_first"
             )
 
             inputs['token_type_ids'] = tokenizer.create_token_type_ids_from_sequences(inputs['input_ids'][1:-1],
-                                                                                       [label_map_seq[relation]])
-            # label_map_seq[relation] 加入关系信息
-            inputs['input_ids'] = inputs['input_ids'] + [label_map_seq[relation], tokenizer.sep_token_id]
+                                                                                       [seq_label2ids[relation]])
+            # label_map_seq[relation] 加入关系信息, 使用seq_label2ids
+            inputs['input_ids'] = inputs['input_ids'] + [seq_label2ids[relation], tokenizer.sep_token_id]
             inputs['attention_mask'] = inputs['attention_mask'] + [1, 1]
 
             # 添加split_text文本的标签
@@ -448,42 +477,56 @@ def convert_examples_to_ner_features(
             split_text_ids = inputs['input_ids']
             # ["O", "B-SUB", "I-SUB", "B-OBJ", "I-OBJ", "Relation"]
             # 默认所有位置都为'O'
-            lable_ner = ['O' for i in range(len(split_text_ids))]
+            label_ner = ['O' for i in range(len(split_text_ids))]
 
             # 标注subject
-            subject_ids = tokenizer.encode(subject, add_special_tokens=False)
-            [start_idx, end_idx] = find_word_in_texts(subject_ids, split_text_ids)
-            if start_idx is None:
-                # logger.info('语料有问题(subject)！%d', ex_index)
-                counter = counter + 1
-                continue
-            lable_ner[start_idx: end_idx] = ['I-SUB' for i in range(len(subject_ids))]
-            lable_ner[start_idx] = 'B-SUB'
+            continue_flag = False
+            for subject in subject_list:
+                subject_ids = tokenizer.encode(subject, add_special_tokens=False)
+                [start_idx, end_idx] = find_word_in_texts(subject_ids, split_text_ids)
+                if start_idx is None:
+                    # logger.info('语料有问题(subject)！%d', ex_index)
+                    continue_flag = True
+                    break
+                label_ner[start_idx: end_idx] = ['I-SUB' for i in range(len(subject_ids))]
+                label_ner[start_idx] = 'B-SUB'
 
+            if continue_flag: continue
             # 标注object
-            object_ids = tokenizer.encode(object_, add_special_tokens=False)
-            [start_idx, end_idx] = find_word_in_texts(object_ids, split_text_ids)
-            if start_idx is None:
-                # logger.info('语料有问题(object)！%d', ex_index)
-                counter = counter + 1
-                continue
-            lable_ner[start_idx: end_idx] = ['I-OBJ' for i in range(len(object_ids))]
-            lable_ner[start_idx] = 'B-OBJ'
+            
+            for object_ in object_list:
+                object_ids = tokenizer.encode(object_, add_special_tokens=False)
+                [start_idx, end_idx] = find_word_in_texts(object_ids, split_text_ids)
+                if start_idx is None:
+                    # logger.info('语料有问题(object)！%d', ex_index)
+                    counter = counter + 1
+                    continue_flag = True
+                    break
+                label_ner[start_idx: end_idx] = ['I-OBJ' for i in range(len(object_ids))]
+                label_ner[start_idx] = 'B-OBJ'
 
+            if continue_flag: continue
             # 标注最后三个字符串，SEP、Relation、SEP
-            lable_ner[-1] = 'O'
-            # lable_ner[-2] = 'Relation'
-            lable_ner[-2] = 'O'
-            lable_ner[-3] = 'O'
+            label_ner[0] = 'CLS'
+            label_ner[-1] = 'SEP'
+            label_ner[-2] = 'Relation'
+            # label_ner[-2] = 'O'
+            label_ner[-3] = 'SEP'
+            
+           
 
-            assert len(lable_ner) == len(inputs['input_ids']) == len(inputs['token_type_ids']) ==\
+            assert len(label_ner) == len(inputs['input_ids']) == len(inputs['token_type_ids']) ==\
                         len(inputs['attention_mask'])
 
             # 关系抽取标签
             label_id_seq = label_map_seq[relation]
 
             # NER标签转换
-            label_id_ner = [label_map_ner[i] for i in lable_ner]
+            label_id_ner = [label_map_ner[i] for i in label_ner]
+            if ex_index == 0:
+                logger.info(example)
+                logger.info(inputs)
+                logger.info(label_id_ner)
 
             features.append(
                 InputFeatures(
@@ -495,27 +538,22 @@ def convert_examples_to_ner_features(
                     words=text
                 )
             )
+            assert len(inputs['input_ids']) <= max_seq_length
 
-    # print('语料有问题句子比例是', str(counter/len(examples)))
+    
+    print('语料有问题句子比例是', str(counter/len(examples)))
+    logger.warning(f"hard to solve total {hard_to_solve} samples. Write code to fix it!")
     return features
 
+
+
 def convert_examples_to_interactive_features(
     examples: List[InputExample],
     labels_seq: List[str],
     labels_ner: List[str],
     max_seq_length: int,
     tokenizer: PreTrainedTokenizer,
-    cls_token_at_end=False,
-    cls_token="[CLS]",
-    cls_token_segment_id=1,
-    sep_token="[SEP]",
-    sep_token_extra=False,
-    pad_on_left=False,
-    pad_token=0,
-    pad_token_segment_id=0,
-    pad_token_label_id=-100,
-    sequence_a_segment_id=0,
-    mask_padding_with_zero=True,
+    rel2id: Dict =None
 ):
     label_map_seq = {label: i for i, label in enumerate(labels_seq)}
     label_map_ner = {label: i for i, label in enumerate(labels_ner)}
@@ -523,114 +561,66 @@ def convert_examples_to_interactive_features(
     features = []
     counter = 0
 
-    def find_word_in_texts(word_ids, texts_ids):
-        length = len(word_ids)
-        for i, W in enumerate(texts_ids):
-            if texts_ids[i] == word_ids[0]:
-                if texts_ids[i: i + length] == word_ids:
-                    return i, i + length
+    def find_word_in_texts(input_ids, entity_ids):
+        length = len(entity_ids)
+        for i, W in enumerate(input_ids):
+            if i+length <= len(input_ids) and input_ids[i: i + length] == entity_ids:
+                return i, i + length
         return None, None
+    
 
     for (ex_index, example) in enumerate(examples):
+        # 用bert分词，转换为token
+        # text = example.text
         if ex_index % 10_000 == 0:
             logger.info("Writing example %d of %d", ex_index, len(examples))
-        #
-        # if ex_index > 1000:
-        #     break
 
-        # seq的输入
-        inputs_seq = tokenizer(
-            example.words,
-            add_special_tokens=True,
-            # return_overflowing_tokens=True,
-        )
-
-        # ner的输入
         text = example.words
-        label_ids_one_text = []
-        features_one_text = []
+        inputs = tokenizer(
+            preprocess(text),
+            add_special_tokens=True,
+            max_length=max_seq_length-2,
+            truncation="longest_first"
+        )
+        bad_flag = False
+        
+        triples = []
+        
         for triple in example.triples:
-            subject = triple[0]
-            relation = triple[1]
-            object_ = triple[2]
+            h, r, t = triple
+            h_ids = tokenizer(preprocess(h), add_special_tokens=False)['input_ids']
+            h_s, h_e = find_word_in_texts(inputs['input_ids'], h_ids)
+            t_ids = tokenizer(preprocess(t), add_special_tokens=False)['input_ids']
+            t_s, t_e = find_word_in_texts(inputs['input_ids'], t_ids)
+            r = rel2id[r]
+            triples.append([h_s,h_e,t_s,t_e,r])
+            if None in triples:
+                bad_flag = True
 
-            inputs = tokenizer(
-                text,
-                add_special_tokens=True,
-                # return_overflowing_tokens=True,
+        if bad_flag: continue
+
+        features.append(
+            InputFeatures_Interactive(
+                input_ids=inputs["input_ids"],
+                attention_mask=inputs["attention_mask"],
+                token_type_ids=inputs["token_type_ids"],
+                triples=triples
             )
-
-            inputs['token_type_ids'] = tokenizer.create_token_type_ids_from_sequences(inputs['input_ids'][1:-1],
-                                                                                       [label_map_seq[relation]])
-            # label_map_seq[relation] 加入关系信息
-            inputs['input_ids'] = inputs['input_ids'] + [ label_map_seq[relation], tokenizer.sep_token_id]
-            inputs['attention_mask'] = inputs['attention_mask'] + [1, 1]
-
-            # 添加split_text文本的标签
-            # B-SUB I-SUB / B-OBJ I-OBJ
-            split_text_ids = inputs['input_ids']
-            # ["O", "B-SUB", "I-SUB", "B-OBJ", "I-OBJ", "Relation"]
-            # 默认所有位置都为'O'
-            lable_ner = ['O' for i in range(len(split_text_ids))]
-
-            # 标注subject
-            subject_ids = tokenizer.encode(subject, add_special_tokens=False)
-            [start_idx, end_idx] = find_word_in_texts(subject_ids, split_text_ids)
-            if start_idx is None:
-                # logger.info('语料有问题(subject)！%d', ex_index)
-                counter = counter + 1
-                continue
-            lable_ner[start_idx: end_idx] = ['I-SUB' for i in range(len(subject_ids))]
-            lable_ner[start_idx] = 'B-SUB'
-
-            # 标注object
-            object_ids = tokenizer.encode(object_, add_special_tokens=False)
-            [start_idx, end_idx] = find_word_in_texts(object_ids, split_text_ids)
-            if start_idx is None:
-                # logger.info('语料有问题(object)！%d', ex_index)
-                counter = counter + 1
-                continue
-            lable_ner[start_idx: end_idx] = ['I-OBJ' for i in range(len(object_ids))]
-            lable_ner[start_idx] = 'B-OBJ'
-
-            # 标注最后三个字符串，SEP、Relation、SEP
-            lable_ner[-1] = 'O'
-            lable_ner[-2] = 'Relation'
-            lable_ner[-3] = 'O'
-
-            # NER标签转换
-            label_id_ner = [label_map_ner[i] for i in lable_ner]
-
-            label_ids_one_text.append(OutputExample(h=subject_ids, r=label_map_seq[relation], t=object_ids))
-            features_one_text.append(
-                InputFeatures_Interactive(
-                    input_ids_seq=inputs_seq["input_ids"],
-                    attention_mask_seq=inputs_seq["attention_mask"],
-                    token_type_ids_seq=inputs_seq["token_type_ids"],
-                    # label_ids_seq=label_id_seq,
-
-                    input_ids_ner=inputs["input_ids"],
-                    attention_mask_ner=inputs["attention_mask"],
-                    token_type_ids_ner=inputs["token_type_ids"],
-                    label_ids_ner=label_id_ner,
-                    # label_ids=label_ids
-                )
-            )
-
-        if len(features_one_text) != 0:
-            for f in features_one_text:
-                f.label_ids = label_ids_one_text
-            features.extend(features_one_text)
-        # else:
-        #     print('23')
-
+        )
+    logger.info(examples[0])
+    logger.info(features[0])
 
     return features
 
-def get_labels_ner() -> List[str]:
-    return ["O", "B-SUB", "I-SUB", "B-OBJ", "I-OBJ", "Relation"]
 
-def get_labels_seq() -> List[str]:
+
+def get_labels_ner() -> List[str]:
+    return ["O", "B-SUB", "I-SUB", "B-OBJ", "I-OBJ", "Relation", "CLS", "SEP"]
+
+def get_labels_seq(args) -> List[str]:
+    # with open(f"{args.data_dir}/rel2id.json", "r") as file:
+    #     t = json.load(file)
+    #     class_label = t.keys()
     class_label = ['Empty', '丈夫', '上映时间', '专业代码', '主持人', '主演', '主角', '人口数量', '作曲', '作者', '作词', '修业年限', '出品公司', '出版社', '出生地', '出生日期','创始人', '制片人', '占地面积', '号', '嘉宾', '国籍', '妻子', '字', '官方语言', '导演', '总部地点', '成立日期', '所在城市', '所属专辑', '改编自', '朝代', '歌手', '母亲', '毕业院校', '民族', '气候', '注册资本', '海拔', '父亲', '目', '祖籍', '简称', '编剧', '董事长', '身高', '连载网站','邮政编码', '面积', '首都']
     return class_label
 
@@ -639,7 +629,6 @@ def openue_data_collator_seq(features):
     max_length = max(max_length)
 
     features_new = []
-    length_seq_label = len(get_labels_seq())
 
     for f in features:
         length = len(f.input_ids)
@@ -652,15 +641,8 @@ def openue_data_collator_seq(features):
         features_['attention_mask'] = f.attention_mask + add_zero  # 补0
         features_['token_type_ids'] = f.token_type_ids + add_zero  # 补0
 
-        label_id_seq = np.zeros(length_seq_label).tolist()
-        for relation_id in f.label_ids_seq:
-            if isinstance(relation_id, int):
-                label_id_seq[relation_id] = 1
-            else:
-                print('error')
-                print(relation_id)
-                print(label_id_seq)
-        features_['label_ids_seq'] = label_id_seq
+        
+        features_['label_ids_seq'] = f.label_ids_seq
         features_new.append(features_)
 
     # 将结构体格式变成dict格式
@@ -670,10 +652,9 @@ def openue_data_collator_seq(features):
     first = features_new[0]
     batch = {}
 
-    batch["label_ids_seq"] = torch.tensor([f["label_ids_seq"] for f in features_new], dtype=torch.float)
 
     for k, v in first.items():
-        if k not in ("label_ids_seq") and v is not None and not isinstance(v, str):
+        if v is not None and not isinstance(v, str):
             if isinstance(v, torch.Tensor):
                 batch[k] = torch.stack([f[k] for f in features_new])
             else:
@@ -722,33 +703,22 @@ def openue_data_collator_ner(features):
     return batch
 
 def openue_data_collator_interactive(features):
-    max_length_seq = [len(f.input_ids_seq) for f in features]
+    max_length_seq = [len(f.input_ids) for f in features]
     max_length_seq = max(max_length_seq)
 
-    max_length_ner = [len(f.input_ids_ner) for f in features]
-    max_length_ner = max(max_length_ner)
-
     features_new = []
     for f in features:
-        length = len(f.input_ids_seq)
+        length = len(f.input_ids)
         distance = max_length_seq - length
         add_zero = [0 for i in range(distance)]
 
         features_ = {}
 
-        features_['input_ids_seq'] = f.input_ids_seq + add_zero  # 补0
-        features_['attention_mask_seq'] = f.attention_mask_seq + add_zero  # 补0
-        features_['token_type_ids_seq'] = f.token_type_ids_seq + add_zero  # 补0
+        features_['input_ids'] = f.input_ids + add_zero  # 补0
+        features_['attention_mask'] = f.attention_mask + add_zero  # 补0
+        features_['token_type_ids'] = f.token_type_ids + add_zero  # 补0
 
-        length = len(f.input_ids_ner)
-        distance = max_length_ner - length
-        add_zero = [0 for i in range(distance)]
-        add_special = [0 for i in range(distance)]
-        features_['input_ids_ner'] = f.input_ids_ner + add_zero  # 补0
-        features_['attention_mask_ner'] = f.attention_mask_ner + add_zero  # 补0
-        features_['token_type_ids_ner'] = f.token_type_ids_ner + add_zero  # 补0
-        features_['label_ids_ner'] = f.label_ids_ner + add_special  # 补0, 为了补齐的最长长度, loss计算中有mask存在会被忽略
-        features_['label_ids'] = f.label_ids
+        features_['triples'] = f.triples
 
         features_new.append(features_)
 
@@ -758,15 +728,15 @@ def openue_data_collator_interactive(features):
 
     first = features_new[0]
     batch = {}
-    # batch["label_ids_seq"] = torch.tensor([f["label_ids_seq"] for f in features], dtype=torch.float)
-    batch["label_ids_ner"] = torch.tensor([f["label_ids_ner"] for f in features_new], dtype=torch.float)
-    batch["label_ids"] = [f["label_ids"] for f in features_new]
 
+    # 这就是完美batch
     for k, v in first.items():
-        if k not in ("label_ids", "label_ids_ner") and v is not None and not isinstance(v, str):
+        if k not in ("triples") and v is not None and not isinstance(v, str):
             if isinstance(v, torch.Tensor):
                 batch[k] = torch.stack([f[k] for f in features_new])
             else:
                 batch[k] = torch.tensor([f[k] for f in features_new], dtype=torch.long)
+        else:
+            batch[k] = [f[k] for f in features_new]
 
     return batch
\ No newline at end of file
